package com.huimv.wine.ws;

import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.fasterxml.jackson.annotation.JsonFormat;
import com.huimv.wine.common.EventWsErrUtil;
import com.huimv.wine.entity.Change;
import com.huimv.wine.entity.Worker;
import com.huimv.wine.entity.vo.WorkerQueryParam;
import com.huimv.wine.entity.vo.WorkerQueryResult;
import com.huimv.wine.entity.vo.WsEvent;
import com.huimv.wine.mapper.ChangeMapper;
import com.huimv.wine.mapper.WorkerMapper;
import com.huimv.wine.utils.*;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Component;
import org.springframework.web.socket.WebSocketMessage;
import org.springframework.web.socket.handler.TextWebSocketHandler;
import sun.nio.cs.ext.MacArabic;

import javax.websocket.*;
import javax.websocket.server.PathParam;
import javax.websocket.server.ServerEndpoint;
import java.io.IOException;
import java.time.Duration;
import java.util.HashMap;
import java.util.Map;

/**
 * websocket接口处理类
 */
@Component
@ServerEndpoint(value = "/worker/{token}")
public class WorkerController extends TextWebSocketHandler {

    /**
     * 连接事件，加入注解
     */



    @OnOpen
    public void onOpen(@PathParam(value = "token") String token, Session session) {
        // 添加到session的映射关系中
        WebsocketWorkerUtil.addSession(token, session);
        getSecrets(session);
    }



    private void getSecrets(Session session) {
        Map map = new HashMap();
        map.put("public",KeyUtil.getClientPublic());
        map.put("private",KeyUtil.getClientPrivate());
        WsEvent wsEvent = new WsEvent("secrets",map);
        WebsocketWorkerUtil.sendMessage(session,wsEvent);
    }

    /**
     * 连接事件，加入注解
     * 用户断开链接
     */
    @OnClose
    public void onClose(@PathParam(value = "token") String token, Session session) {
        // 删除映射关系
        WebsocketWorkerUtil.removeSession(token);
    }

    /**
     * 当接收到用户上传的消息
     */
    @OnMessage
    public void onMessage(@PathParam(value = "token") String token, Session session, String message) {
        String msg = "[" + token + "]:" + message;
        System.out.println("接收到信息：" + msg);

        WorkerMapper workerMapper = SpringContextUtil.getBean(WorkerMapper.class);
        ChangeMapper changeMapper = SpringContextUtil.getBean(ChangeMapper.class);
        RedisTemplate redisTemplate = SpringContextUtil.getBean(RedisTemplate.class);
        JSONObject jsonObject = JSON.parseObject(message);
        String event = jsonObject.getString("event");
        JSONObject data = jsonObject.getJSONObject("data");

        if ("login".equals(event)){
            login(workerMapper,session,token,data);
        }else if ("pin".equals(event)){
            keepAlive(session);
        }else {
            if (!redisTemplate.hasKey(token)) {
                WebsocketWorkerUtil.sendMessage(session,new WsEvent("tokenExpired",null));
                return;
            }
            String worker = (String)redisTemplate.opsForValue().get(token);
            switch (event){
                case "query":
                    query(workerMapper,session,worker, data);
                    break;
                case "detail":
                    detail(workerMapper,session,data);
                    break;
                case "ready":
                    ready(changeMapper,session, jsonObject.getInteger("data"));
                    break;
                case "finish":
                    finish(changeMapper,session,jsonObject.getInteger("data"));
                    break;
                default:
                    WebsocketWorkerUtil.sendMessage(session, EventWsErrUtil.getWsErr("unrecognized event"));
                    break;
            }
        }
    }

    private void finish(ChangeMapper changeMapper, Session session, Integer id) {
        Change change = changeMapper.selectById(id);
        if (ObjectUtil.isEmpty(change)){
            WebsocketWorkerUtil.sendMessage(session,new WsEvent("finishResult",Result.fail("订单不存在")));
        }
        if (change.getStatus() != 1){
            WebsocketWorkerUtil.sendMessage(session,new WsEvent("finishResult",Result.fail("请先完成上酒动作")));
        }

    }

    private void ready(ChangeMapper changeMapper, Session session, Integer id) {
        Change change = changeMapper.selectById(id);
        if (ObjectUtil.isEmpty(change)){
            WebsocketWorkerUtil.sendMessage(session,new WsEvent("finishResult",Result.fail("订单不存在")));
        }
        if (change.getStatus() != 0){
            WebsocketWorkerUtil.sendMessage(session,new WsEvent("finishResult",Result.fail("订单状态异常")));
        }
    }

    private void detail(WorkerMapper WorkerMapper, Session session, JSONObject data) {

    }

    private void query(WorkerMapper WorkerMapper, Session session,String worker, JSONObject data) {
        WorkerQueryParam workerQueryParam = JSONObject.toJavaObject(data, WorkerQueryParam.class);
        Page<WorkerQueryResult> page = new Page<>(workerQueryParam.getPage(),workerQueryParam.getLimit());
        String like = "%"+ workerQueryParam.getCond() +"%";
        Page<WorkerQueryResult> workerQueryResultPage = WorkerMapper.ChangesQueryForWorker(page,workerQueryParam.getStatus(),like,worker);
        WebsocketWorkerUtil.sendMessage(session,new WsEvent("queryResult",Result.success(workerQueryResultPage)));
    }

    private void login(WorkerMapper WorkerMapper, Session session, String token, JSONObject data) {
        RedisTemplate redisTemplate = SpringContextUtil.getBean(RedisTemplate.class);

        String account = data.getString("account");
        String password = data.getString("password");

        String serverPrivate = KeyUtil.getServerPrivate();
        String realPassword = RSAUtil.decrypt1(password, serverPrivate);
        Worker worker = WorkerMapper.login(account,realPassword);
        if (ObjectUtil.isEmpty(worker)){
            WsEvent wsEvent = new WsEvent("loginResult",Result.fail("用户名或密码错误"));
            WebsocketWorkerUtil.sendMessage(session,wsEvent);
        }else {
            redisTemplate.opsForValue().set(token,worker.getId(), Duration.ofDays(7));
            WsEvent wsEvent = new WsEvent("loginResult",Result.success(token));
            WebsocketWorkerUtil.sendMessage(session,wsEvent);
        }

    }

    private void keepAlive(Session session) {
        WebsocketSellerUtil.sendMessage(session,new WsEvent("pon",null));
    }

    private void loginSocket(String message, Session session) {

    }

    /**
     * 处理用户活连接异常
     *
     * @param session
     * @param throwable
     */
    @OnError
    public void onError(Session session, Throwable throwable) {
        try {
            session.close();
        } catch (IOException e) {
            e.printStackTrace();
        }
        throwable.printStackTrace();
    }
}
